import { __decorate } from "tslib";
import { Combobox as FASTComboBox } from '@microsoft/fast-components';
import { DOM, Observable, observable } from '@microsoft/fast-element';
import { foundationComboboxStyles as styles } from './combobox.styles';
import { foundationComboboxTemplate as template } from './combobox.template';
export const wasClickOutsideElement = (event, target) => !event.composedPath().includes(target);
export class Combobox extends FASTComboBox {
    constructor() {
        super();
        this.open = false;
        this.clickOutside = (e) => this.handleClickOutside(e);
        this.addEventListener('focus', this.handleFocus);
    }
    handleFocus(event) {
        this.open = !this.open;
        DOM.queueUpdate(() => {
            const input = this.shadowRoot.querySelector('input');
            input.focus();
        });
    }
    findCaseInsensitiveIndex(text) {
        if (this.options) {
            return this.options.findIndex((el) => el.text === text);
        }
        return -1;
    }
    get value() {
        Observable.track(this, 'value');
        return this['_value'];
    }
    set value(next) {
        var _a, _b, _c;
        const prev = `${this['_value']}`;
        if (this.$fastController.isConnected && this.options) {
            const selectedIndex = this.findCaseInsensitiveIndex(next);
            const prevSelectedValue = (_a = this.options[this.selectedIndex]) === null || _a === void 0 ? void 0 : _a.text;
            const nextSelectedValue = (_b = this.options[selectedIndex]) === null || _b === void 0 ? void 0 : _b.text;
            this.selectedIndex =
                prevSelectedValue !== nextSelectedValue ? selectedIndex : this.selectedIndex;
            next = ((_c = this.firstSelectedOption) === null || _c === void 0 ? void 0 : _c.text) || next;
        }
        if (prev !== next) {
            this['_value'] = next;
            super.valueChanged(prev, next);
            Observable.notify(this, 'value');
        }
    }
    handleClickOutside(event) {
        if (wasClickOutsideElement(event, this)) {
            this.open = false;
        }
    }
}
__decorate([
    observable
], Combobox.prototype, "open", void 0);
// explicitly declaring for extensibility
export const foundationComboboxShadowOptions = undefined;
export const defaultComboboxConfig = {
    indicator: `
        <svg
            class="combo-box-indicator"
            part="combo-box-indicator"
            viewBox="0 0 12 7"
            xmlns="http://www.w3.org/2000/svg"
        >
            <path
                d="M11.85.65c.2.2.2.5 0 .7L6.4 6.84a.55.55 0 01-.78 0L.14 1.35a.5.5 0 11.71-.7L6 5.8 11.15.65c.2-.2.5-.2.7 0z"
            />
        </svg>
    `,
};
/**
 * The Foundation Combo-box
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-combo-box\>
 */
export const foundationCombobox = Combobox.compose(Object.assign({ baseName: 'combobox', template,
    styles, shadowOptions: foundationComboboxShadowOptions }, defaultComboboxConfig));
